#ifndef B3_GJK_H
#define B3_GJK_H

#include <Bounce\Common\b3Geometry.h>

class b3GJKProxy;
struct b3SimplexCache;

struct b3SimplexVertex
{
	b3Vec3 pointA; // support vertex on proxy A
	b3Vec3 pointB; // support vertex on proxy B
	b3Vec3 point; // minkowski vertex
	float32 weight; // barycentric coordinate for point
	u32 indexA; // support A index
	u32 indexB; // support B index
};

struct b3Simplex
{
	b3SimplexVertex m_vertices[4];
	u32 m_count;

	b3Vec3 GetSearchDirection(const b3Vec3& Q) const;
	b3Vec3 GetClosestPoint() const;
	void GetClosestPoints(b3Vec3* pA, b3Vec3* pB) const;

	void Solve2(const b3Vec3& Q);
	void Solve3(const b3Vec3& Q);
	void Solve4(const b3Vec3& Q);

	// Cache
	void ReadCache(const b3SimplexCache* cache,
		const b3Transform& xfA, const b3GJKProxy& proxyA,
		const b3Transform& xfB, const b3GJKProxy& proxyB);
	void WriteCache(b3SimplexCache* cache) const;
	float32 GetMetric() const;
};

// The output of the GJK algorithm.
// It contains the closest points between two proxies 
// and their euclidean distance.
struct b3GJKOutput 
{
	b3Vec3 pointA; // closest point on proxy A
	b3Vec3 pointB; // closest point on proxy B
	float32 distance; // euclidean distance between the closest points
	u32 iterations; // number of GJK iterations
};

// Find the closest points and distance between two proxies.
// If the distance is zero then the proxies are overlapping.
b3GJKOutput b3GJK(const b3Transform& xfA, const b3GJKProxy& proxyA, 
	const b3Transform& xfB, const b3GJKProxy& proxyB);

#endif