#ifndef B3_GJK_CACHE_H
#define B3_GJK_CACHE_H

#include <Bounce\Collision\GJK\b3GJK.h>

// A simplex used to improve the performance 
// of the GJK when called more than once. 
// Make sure to set cache.count to zero before 
// calling the GJK for the first time.
struct b3SimplexCache
{
	float32 metric; // length or area or volume
	u32 iterations; // number of GJK iterations
	u16 count; // number of support vertices
	u8 indexA[4]; // support vertices on proxy A
	u8 indexB[4]; // support vertices on proxy B
};

// Find the closest points and distance between two proxies. 
// Assumes a simplex is given for increasing the performance of 
// the GJK when called more than once.
b3GJKOutput b3GJK(const b3Transform& xfA, const b3GJKProxy& proxyA,
				  const b3Transform& xfB, const b3GJKProxy& proxyB,
				  bool applyRadius, b3SimplexCache* cache);

struct b3GJKFeaturePair
{
	enum Type
	{
		e_unknown = 0,
		e_vertex = 1,
		e_edge = 2,
		e_face = 3
	};

	Type typeA; // number of vertices on proxy A
	Type typeB; // number of vertices on proxy B
	u32 indexA[3]; // vertices on proxy A
	u32 indexB[3]; // vertices on proxy B
};

// Get the vertices of the features that the closest points between two 
// GJK proxies are contained on given a cached simplex.
// The GJK must have been called using the pair of proxies and 
// cache.count must be < 4, that is, the proxies must not be overlapping.
b3GJKFeaturePair b3GetFeaturePair(const b3SimplexCache& cache);

#endif