#ifndef B3_GJK_PROXY_H
#define B3_GJK_PROXY_H

#include <Bounce\Common\Math\b3Vec3.h>

// A GJK proxy encapsulates any convex hull to be used by the GJK.
class b3GJKProxy
{
public:
	b3GJKProxy() : m_vertices(nullptr), m_count(0), m_radius(0.0f) { }

	// Get the number of vertices of this proxy.
	u32 GetVertexCount() const;

	// Read an indexed vertex from this proxy.
	const b3Vec3& GetVertex(u32 index) const;

	// Get the support vertex index in a given direction.
	u32 GetSupportIndex(const b3Vec3& d) const;

	// Get the support vertex in a given direction.
	const b3Vec3& GetSupportVertex(const b3Vec3& d) const;

	b3Vec3 m_buffer[3]; // for childs
	const b3Vec3* m_vertices;
	u32 m_count;
	float32 m_radius;
};

inline u32 b3GJKProxy::GetVertexCount() const
{
	return m_count;
}

inline const b3Vec3& b3GJKProxy::GetVertex(u32 index) const
{
	B3_ASSERT(0 <= index && index < m_count);
	return m_vertices[index];
}

inline u32 b3GJKProxy::GetSupportIndex(const b3Vec3& d) const
{
	u32 maxIndex = 0;
	float32 maxProjection = b3Dot(d, m_vertices[maxIndex]);
	for (u32 i = 1; i < m_count; ++i)
	{
		float32 projection = b3Dot(d, m_vertices[i]);
		if (projection > maxProjection)
		{
			maxIndex = i;
			maxProjection = projection;
		}
	}
	return maxIndex;
}

inline const b3Vec3& b3GJKProxy::GetSupportVertex(const b3Vec3& d) const
{
	u32 maxIndex = 0;
	float32 maxProjection = b3Dot(d, m_vertices[maxIndex]);
	for (u32 i = 1; i < m_count; ++i)
	{
		float32 projection = b3Dot(d, m_vertices[i]);
		if (projection > maxProjection)
		{
			maxIndex = i;
			maxProjection = projection;
		}
	}
	return m_vertices[maxIndex];
}

#endif
